import 'dart:io';
import 'package:fish_redux/fish_redux.dart';
import 'package:flutter/cupertino.dart' hide Action;
import 'package:flutter_livepush_demo/common/CommonUtils.dart';
import 'package:flutter_livepush_demo/common/InteractiveUrlUtils.dart';
import 'package:flutter_aliplayer/flutter_aliplayer.dart';
import 'package:flutter_livepush_demo/page/interactive/interactive_mode/interactive_mode_state.dart';
import 'package:flutter_livepush_plugin/live_base.dart';
import 'package:flutter_livepush_plugin/live_push_def.dart';
import 'package:flutter_livepush_plugin/live_pusher.dart';
import 'package:flutter_livepush_plugin/interactive/live_player.dart';
import 'package:flutter_livepush_plugin/interactive/live_player_config.dart';
import 'package:flutter_livepush_plugin/interactive/live_transcoding_config.dart';
import 'package:flutter_aliplayer/flutter_aliplayer_factory.dart';
import 'interactive_mode_action.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';

Effect<InteractiveModeState> buildEffect() {
  return combineEffects(<Object, Effect<InteractiveModeState>>{
    Lifecycle.initState: _init,
    Lifecycle.dispose: _dispose,
    InteractiveModeAction.startPreview: _startPreview,
    InteractiveModeAction.startLink: _startLink,
    InteractiveModeAction.stopLink: _stopLink,
    InteractiveModeAction.clickSwitchCamera: _clickSwitchCamera,
    InteractiveModeAction.clickMute: _clickMute,
    InteractiveModeAction.clickSwitchSpeaker: _clickSwitchSpeaker,
    InteractiveModeAction.clickNext: _clickNext,
    InteractiveModeAction.goBackPage: _goBackPage,
  });
}

late AlivcBase _alivcBase;
late AlivcLivePusher _alivcLivePusher;
late AlivcLivePlayer _alivcLivePlayer;
late AlivcLivePlayConfig _alivcLivePlayConfig;
late FlutterAliplayer _aliPlayer;

Future<void> _init(Action action, Context<InteractiveModeState> ctx) async {
  _registerSDK(action, ctx);
  _setRTCPusher(action, ctx);
  _setRTCPlayer(action, ctx);

  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    FlutterAliPlayerFactory.initLicenseServiceForIOS();
    _setCDNPlayer(action, ctx);
  }

  ctx.dispatch(InteractiveModeActionCreator.pageDidAppear(true));
}

Future<void> _registerSDK(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcBase = AlivcBase.init();
  _alivcBase.registerSDK();
  _alivcBase.setObserver();
  _alivcBase.setOnLicenceCheck((result, reason) {
    if (result != AlivcLiveLicenseCheckResultCode.success) {
      Fluttertoast.showToast(
        msg: reason,
        gravity: ToastGravity.CENTER,
      );
    }
  });
}

Future<void> _setRTCPusher(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePusher = AlivcLivePusher.init();
  ctx.state.pusherConfig.setLivePushMode(AlivcLivePushMode.interactive);
  _alivcLivePusher.initLivePusher();
  _alivcLivePusher.setErrorDelegate();
  _alivcLivePusher.setInfoDelegate();
  _alivcLivePusher.setNetworkDelegate();

  /// SDK错误回调
  _alivcLivePusher.setOnSDKError((errorCode, errorDescription) {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_sdk_error,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 系统错误回调
  _alivcLivePusher.setOnSystemError((errorCode, errorDescription) {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_system_error,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 开始预览回调
  _alivcLivePusher.setOnPreviewStarted(() {});

  /// 停止预览回调
  _alivcLivePusher.setOnPreviewStoped(() {});

  /// 渲染第一帧回调
  _alivcLivePusher.setOnFirstFramePreviewed(() {});

  /// 推流开始回调
  _alivcLivePusher.setOnPushStarted(() {});

  /// 摄像头推流暂停回调
  _alivcLivePusher.setOnPushPaused(() {});

  /// 摄像头推流恢复回调
  _alivcLivePusher.setOnPushResumed(() {});

  /// 重新推流回调
  _alivcLivePusher.setOnPushRestart(() {});

  /// 推流停止回调
  _alivcLivePusher.setOnPushStoped(() {});

  /// 推流链接失败
  _alivcLivePusher.setOnConnectFail((errorCode, errorDescription) {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_connect_failed_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 网络恢复
  _alivcLivePusher.setOnConnectRecovery(() {});

  /// 连接被断开
  _alivcLivePusher.setOnConnectionLost(() {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_connection_lost_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 网络差回调
  _alivcLivePusher.setOnNetworkPoor(() {
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_network_slow_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 重连失败回调
  _alivcLivePusher.setOnReconnectError((errorCode, errorDescription) {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_reconnect_fail_log,
      gravity: ToastGravity.CENTER,
    );
  });

  /// 重连开始回调
  _alivcLivePusher.setOnReconnectStart(() {});

  /// 重连成功回调
  _alivcLivePusher.setOnReconnectSuccess(() {});

  /// 发送数据超时
  _alivcLivePusher.setOnSendDataTimeout(() {
    ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
        InteractivePushStatus.error));
    Fluttertoast.showToast(
      msg: AppLocalizations.of(ctx.context)!.camerapush_send_data_timeout_log,
      gravity: ToastGravity.CENTER,
    );
  });
}

Future<void> _startRTCPush(
    Action action, Context<InteractiveModeState> ctx) async {
  String rtcPushURL = InteractiveUrlUtils.generateInteractivePushUrl(
      ctx.state.roomId, ctx.state.userId);
  print("rtcPushURL:${rtcPushURL}");
  if (rtcPushURL.isEmpty) {
    Fluttertoast.showToast(
        msg: "Start Push Error", gravity: ToastGravity.CENTER);
    return;
  }

  _alivcLivePusher.startPreview();
  _alivcLivePusher.startPushWithURL(rtcPushURL);

  ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
      InteractivePushStatus.pushing));
}

Future<void> _stopRTCPush(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePusher.stopPush();
  _alivcLivePusher.stopPreview();
  ctx.dispatch(InteractiveModeActionCreator.updatePushStatus(
      InteractivePushStatus.stop));
}

Future<void> _setRTCPlayer(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePlayer = AlivcLivePlayer.init();
  _alivcLivePlayer.bindPlayCofig();
  _alivcLivePlayConfig = AlivcLivePlayConfig.init();
  _alivcLivePlayConfig.setRenderMode(AlivcLivePlayRenderMode.auto);

  _alivcLivePlayer.initLivePlayer();
  _alivcLivePlayer.setLivePlayerDelegate();

  _alivcLivePlayer.setOnError((errorCode, errorMessage) {
    if (errorCode == AlivcLivePlayerError.streamNotFound) {
      _stopRTCPlay(action, ctx);
      if (ctx.state.identityTypeEnum != InteractiveIdentityTypeEnum.audience) {
        ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(false));
      }
      Fluttertoast.showToast(
          msg: AppLocalizations.of(ctx.context)!.interactive_not_pull_stream,
          gravity: ToastGravity.CENTER);
    } else if (errorCode == AlivcLivePlayerError.streamStopped) {
      _stopRTCPlay(action, ctx);
      if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive) {
        if (ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.anchor) {
          ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(false));
          Fluttertoast.showToast(
              msg: AppLocalizations.of(ctx.context)!.interactive_viewer_left,
              gravity: ToastGravity.CENTER);
        } else {
          _stopRTCPush(action, ctx);
          ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(false));
          Fluttertoast.showToast(
              msg: AppLocalizations.of(ctx.context)!.interactive_streamer_left,
              gravity: ToastGravity.CENTER);
          _goBackPage(action, ctx);
        }
      } else {
        ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(false));
        Fluttertoast.showToast(
            msg: AppLocalizations.of(ctx.context)!.interactive_streamer_left,
            gravity: ToastGravity.CENTER);
      }
    } else {
      ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
          InteractivePullStatus.error));
      Fluttertoast.showToast(msg: errorMessage, gravity: ToastGravity.CENTER);
    }
  });
  _alivcLivePlayer.setOnPlayStarted(() {});
  _alivcLivePlayer.setOnPlayStoped(() {});
  _alivcLivePlayer.setOnFirstVideoFrameDrawn(() {});
}

Future<void> _startRTCPlay(
    Action action, Context<InteractiveModeState> ctx) async {
  String rtcPlayURL = InteractiveUrlUtils.generateInteractivePullUrl(
      ctx.state.customerInfo.roomId, ctx.state.customerInfo.userId);
  println("rtcPlayURL:$rtcPlayURL}");
  if (rtcPlayURL.isEmpty) {
    Fluttertoast.showToast(
        msg:
            AppLocalizations.of(ctx.context)!.interactive_input_pull_stream_url,
        gravity: ToastGravity.CENTER);
    return;
  }

  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive) {
    if (Platform.isAndroid) {
      _alivcLivePlayConfig.setIsFullScreen(
          ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience);
    }
  }

  _alivcLivePlayer.setPlayView();
  _alivcLivePlayer.startPlayWithURL(rtcPlayURL);

  if (ctx.state.identityTypeEnum != InteractiveIdentityTypeEnum.audience) {
    _setMixStream(action, ctx, true);
  }

  ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
      InteractivePullStatus.pulling));
}

Future<void> _stopRTCPlay(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePlayer.stopPlay();

  if (ctx.state.identityTypeEnum != InteractiveIdentityTypeEnum.audience) {
    _setMixStream(action, ctx, false);
  }

  ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
      InteractivePullStatus.stop));
}

Future<void> _setMixStream(
    Action action, Context<InteractiveModeState> ctx, bool isNeed) async {
  if (isNeed) {
    AlivcLiveTranscodingConfig _liveTranscodingConfig =
        AlivcLiveTranscodingConfig.init();
    _alivcLivePusher.bindLiveMixTranscodingConfig();

    Size pushResolution = await ctx.state.pusherConfig.getPushResolution();

    AlivcLiveMixStream pusherMixStream = AlivcLiveMixStream();
    pusherMixStream.userId = ctx.state.userId;
    pusherMixStream.x = 0;

    if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive) {
      pusherMixStream.y = 0;
      pusherMixStream.width = pushResolution.width.round();
      pusherMixStream.height = pushResolution.height.round();
    } else {
      double pushMixStreamWidth = pushResolution.width / 2.0;
      double pushMixStreamHeight =
          (pushMixStreamWidth * pushResolution.height) / pushResolution.width;
      pusherMixStream.y =
          ((pushResolution.height - pushMixStreamHeight) / 2.0).round();
      pusherMixStream.width = pushMixStreamWidth.round();
      pusherMixStream.height = pushMixStreamHeight.round();
    }

    pusherMixStream.zOrder = 1;

    print(
        "pusherMixStream userId:${pusherMixStream.userId}, x:${pusherMixStream.x}, y:${pusherMixStream.y}, width:${pusherMixStream.width}, height:${pusherMixStream.height}");

    AlivcLiveMixStream pullerMixStream = AlivcLiveMixStream();
    pullerMixStream.userId = ctx.state.customerInfo.userId;

    if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive) {
      int pullerMixStreamWidth = 90;
      int pullerMixStreamHeight = 160;

      pullerMixStream.x =
          (pushResolution.width - pullerMixStreamWidth - 35).round();
      pullerMixStream.y =
          (pushResolution.height - pullerMixStreamHeight - 35).round();
      pullerMixStream.width = 90;
      pullerMixStream.height = 160;
    } else {
      double pusherMixStreamWidth = pushResolution.width / 2.0;
      double pusherMixStreamHeight =
          (pusherMixStreamWidth * pushResolution.height) / pushResolution.width;

      pullerMixStream.x = (pusherMixStreamWidth + 1).round();
      pullerMixStream.y =
          ((pushResolution.height - pusherMixStreamHeight) / 2.0).round();
      pullerMixStream.width = pusherMixStreamWidth.round();
      pullerMixStream.height = pusherMixStreamHeight.round();
    }

    pullerMixStream.zOrder = 2;
    print(
        "pullerMixStream userId:${pullerMixStream.userId}, x:${pullerMixStream.x}, y:${pullerMixStream.y}, width:${pullerMixStream.width}, height:${pullerMixStream.height}");

    _liveTranscodingConfig.setMixStreams([pusherMixStream, pullerMixStream]);
  }
  _alivcLivePusher.setLiveMixTranscodingConfig(isNeed);
}

Future<void> _setCDNPlayer(
    Action action, Context<InteractiveModeState> ctx) async {
  _aliPlayer = FlutterAliPlayerFactory.createAliPlayer();
  _aliPlayer.setAutoPlay(true);

  _aliPlayer.setOnCompletion((playerId) {});
  _aliPlayer.setOnError((errorCode, errorExtra, errorMsg, playerId) async {
    String errMsg = "Pull Play Error:[${errorCode}]${errorMsg}";
    Fluttertoast.showToast(msg: errMsg, gravity: ToastGravity.CENTER);
    ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
        InteractivePullStatus.error));

    _aliPlayer.stop();
    _aliPlayer.clearScreen();
    await Future.delayed(Duration(milliseconds: 2000));
    _aliPlayer.prepare();
  });
}

Future<void> _startCDNPull(
    Action action, Context<InteractiveModeState> ctx) async {
  bool audioOnly = await ctx.state.pusherConfig.getAudioOnly();
  String cdnPlayURL = InteractiveUrlUtils.generateCDNPullUrl(
      ctx.state.customerInfo.roomId, ctx.state.customerInfo.userId, audioOnly);
  print("cdnPlayURL:${cdnPlayURL}");
  if (cdnPlayURL.isEmpty) {
    Fluttertoast.showToast(
        msg:
            AppLocalizations.of(ctx.context)!.interactive_input_pull_stream_url,
        gravity: ToastGravity.CENTER);
    return;
  }

  _aliPlayer.setPlayerView(ctx.state.cdnPlayerViewId);
  _aliPlayer.setUrl(cdnPlayURL);
  _aliPlayer.prepare();

  ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
      InteractivePullStatus.pulling));
}

Future<void> _stopCDNPull(
    Action action, Context<InteractiveModeState> ctx) async {
  _aliPlayer.stop();
  _aliPlayer.clearScreen();

  ctx.dispatch(InteractiveModeActionCreator.updatePullStatus(
      InteractivePullStatus.stop));
}

Future<void> _startPreview(
    Action action, Context<InteractiveModeState> ctx) async {
  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    _startCDNPull(action, ctx);
  } else {
    _startRTCPush(action, ctx);
  }
}

Future<void> _startLink(
    Action action, Context<InteractiveModeState> ctx) async {
  ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(true));
  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    _stopCDNPull(action, ctx);
    await Future.delayed(const Duration(milliseconds: 500));
    _startRTCPush(action, ctx);
    _startRTCPlay(action, ctx);
  } else {
    await Future.delayed(const Duration(milliseconds: 500));
    _startRTCPlay(action, ctx);
  }
}

Future<void> _stopLink(Action action, Context<InteractiveModeState> ctx) async {
  ctx.dispatch(InteractiveModeActionCreator.clickBtnLink(false));
  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    _stopRTCPush(action, ctx);

    await Future.delayed(Duration(milliseconds: 500));
    _stopRTCPlay(action, ctx);

    await Future.delayed(Duration(milliseconds: 500));
    _startCDNPull(action, ctx);
  } else {
    _stopRTCPlay(action, ctx);
  }
}

Future<void> _clickMute(
    Action action, Context<InteractiveModeState> ctx) async {
  bool isMute = action.payload ?? false;
  _alivcLivePusher.setMute(isMute);
  ctx.dispatch(InteractiveModeActionCreator.onClickMute(isMute));
}

Future<void> _clickSwitchCamera(
    Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePusher.switchCamera();
}

Future<void> _clickSwitchSpeaker(
    Action action, Context<InteractiveModeState> ctx) async {
  bool enable = await _alivcLivePusher.isEnableSpeakerphone();
  _alivcLivePusher.enableSpeakerphone(enable ? false : true);
}

Future<void> _clickNext(
    Action action, Context<InteractiveModeState> ctx) async {
  var interactiveInfoState = ctx.state;
}

Future<void> _goBackPage(
    Action action, Context<InteractiveModeState> ctx) async {
  _destory(action, ctx);
  Navigator.of(ctx.context).pop();
}

Future<void> _destory(Action action, Context<InteractiveModeState> ctx) async {
  _alivcLivePusher.destroy();

  _alivcLivePlayer.stopPlay();

  if (ctx.state.typeEnum == InteractiveTypeEnum.interactiveLive &&
      ctx.state.identityTypeEnum == InteractiveIdentityTypeEnum.audience) {
    _aliPlayer.stop();
    _aliPlayer.destroy();
  }
}

Future<void> _dispose(Action action, Context<InteractiveModeState> ctx) async {}
